// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

#include "NimCore.h"

#include <flutter/encodable_value.h>
#include <flutter/method_channel.h>
#include <flutter/plugin_registrar_windows.h>
#include <flutter/standard_method_codec.h>

#include "services/FLTAudioRecorderService.h"
#include "services/FLTAuthService.h"
#include "services/FLTChatRoomService.h"
#include "services/FLTEventSubscribeService.h"
#include "services/FLTInitializeService.h"
#include "services/FLTMessageService.h"
#include "services/FLTNOSService.h"
#include "services/FLTPassThroughService.h"
#include "services/FLTSystemNotificationService.h"
#include "services/FLTTeamService.h"
#include "services/FLTUserService.h"
#include "FLTConvert.h"

const std::string kFLTNimCoreService = "serviceName";

NimCore::NimCore() {
    addService(new FLTInitializeService());
}

NimCore::~NimCore() {
}

void NimCore::regService() {
    addService(new FLTAuthService());
    addService(new FLTAudioRecorderService());
    addService(new FLTEventSubscribeService());
    addService(new FLTMessageService());
    addService(new FLTNOSService());
    addService(new FLTPassThroughService());
    addService(new FLTSystemNotificationService());
    addService(new FLTUserService());
    addService(new FLTChatRoomService());
    addService(new FLTTeamService());
}

void NimCore::addService(FLTService* service) {
    m_services[service->getServiceName()] = service;
}

FLTAuthService* NimCore::getFLTAuthService() {
    return dynamic_cast<FLTAuthService*>(getService("AuthService"));
}

FLTMessageService* NimCore::getFLTMessageService() {
    return dynamic_cast<FLTMessageService*>(getService("MessageService"));
}

FLTService* NimCore::getService(const std::string& serviceName) {
    auto service = m_services.find(serviceName);
    if (m_services.end() == service) {
        return nullptr;
    }

    return service->second;
}

void NimCore::onMethodCall(const std::string& method, const flutter::EncodableMap* arguments,
                           std::shared_ptr<flutter::MethodResult<flutter::EncodableValue>> result) {
    if (nullptr == arguments) {
        if (result) {
            result->NotImplemented();
        }
        return;
    }

    auto serviceName_iter = arguments->find(flutter::EncodableValue("serviceName"));
    if (serviceName_iter != arguments->end() && !serviceName_iter->second.IsNull()) {
        std::string serviceName = std::get<std::string>(serviceName_iter->second);
        auto* service = getService(serviceName);
        if (service) {
            YXLOG_API(Info) << "serviceName: " << service->getServiceName() << ", methodName: " << method
                            << ", arguments: " << Convert::getInstance()->getStringFormMapForLog(arguments) << YXLOGEnd;
            service->onMethodCalled(method, arguments, result);
            return;
        }
    } else {
        YXLOG_API(Warn) << "serviceName not found, methodName: " << method << YXLOGEnd;
    }

    if (result) {
        result->NotImplemented();
    }
}

void NimCore::setMethodChannel(NimMethodChannel* channel) {
    m_channel = channel;
}

NimCore::NimMethodChannel* NimCore::getMethodChannel() {
    return m_channel;
}

void NimCore::setAppkey(const std::string& appkey) {
    m_appKey = appkey;
}

std::string NimCore::getAppkey() const {
    return m_appKey;
}

void NimCore::setLogDir(const std::string& logDir) {
    m_logDir = logDir;
}

std::string NimCore::getLogDir() const {
    return m_logDir;
}
